import click
from . import git
from . import common

@click.group()
def cli():
    pass

@cli.command()
@click.option("--merged-on-branch", "branch", metavar="BRANCH", default="master", help="which branch to check if merged on it", show_default=True)
@click.option("--local/--no-local", default=False, help="Apply on local branches or not", show_default=True)
@click.option("--remote/--no-remote", default=True, help="Apply on remote branches or not", show_default=True)
@click.option("--dry-run", is_flag=True, default=False, help="List only no actual deletion will be done", show_default=True)
@click.option("--skip-fetch", is_flag=True, default=False, help="Skip fetching the new updates before listing/deleting branches", show_default=True)
@click.option("--yes", "confirmation", is_flag=True, default=False, help="Continue without confirmation BE CAREFUL!!!", show_default=True)
def delete_all_merged_branches(branch, local, remote, dry_run, skip_fetch, confirmation):
    ''' delete all banches merged on given branch '''

    if not skip_fetch:
        git.fetch()

    merged_branches = git.get_merged_branches(branch=branch)
    common.print_seperator()
    common.print_list(merged_branches)
    common.print_seperator()
    click.secho(f"number of merged branches on {branch}: {len(merged_branches)}", bold=True, fg="blue")
    common.print_seperator()

    delete_message = "local and remote" if local and remote else "local" if local else "remote" if remote else ""
    if (local or remote) and len(merged_branches) > 0 and (confirmation or click.confirm(f"Are you sure you want to delete {delete_message} branches?")):
        remote_name = git.get_remote_name()
        for b in merged_branches:
            if dry_run:
                click.secho(f"'{b}' will be deleted if not --dry-run", fg="red")
            else:
                git.delete_branch(branch=b, local=local, remote=remote, remote_name=remote_name)
            
        common.print_seperator()
        print("🎉❌ BRANCHES DELETED ❌🎉")
    else:
        print(" 🚫mission abort 🚫")


import sys
from .read_input_helper import getch

def print_seperator(sep="_", count=120):
    print(sep*count)

def print_list(list:list):
    print(*list, sep="\n")

def strip(string):
    return string.strip()

def printf(*values,  sep=' ', end='\n', file=sys.stdout, flush=False, silent=False):
    if not silent:
        print(values=values, sep=sep, end=end, file=file, flush=flush)

def get_list_from_string(string, seperator="\n"):
    output_list = [item for item in string.split(seperator) if item != ""]
    return list(map(strip, output_list))

def replace_every_string(current_string, with_string, in_list:list):
    return list(map(lambda x: x.replace(current_string, with_string), in_list))


import subprocess
from . import common

def fetch():
    print("### Updating from remote ⏬⏬")
    subprocess.run(["git fetch -p"], shell=True)

def checkout(branch):
    print(f"\nchecking out {branch}")
    subprocess.run(["git", "checkout", branch])

def pull(branch):
    checkout(branch)
    merge()

def get_remote_name():
    return subprocess.run(["git remote"], shell=True, capture_output=True, universal_newlines=True).stdout.strip()

def get_current_branch():
    return subprocess.run(["git symbolic-ref --short -q HEAD"], shell=True, capture_output=True, universal_newlines=True).stdout.strip()

def get_merged_branches(branch="master"):
    # checkout(branch)
    merged_branches = common.get_list_from_string(subprocess.run([f"git branch -r --merged {branch} --no-contains {branch} | egrep -v \"(^\*|master)\""], shell=True, capture_output=True, universal_newlines=True).stdout.replace(f"{get_remote_name()}/", ""))
    return merged_branches

def get_branches_deleted_remotely():
    return common.get_list_from_string(subprocess.run(["git for-each-ref --format '%(refname) %(upstream:track)' refs/heads | awk '$2 == \"[gone]\" {sub(\"refs/heads/\", \"\", $1); print $1}'"], shell=True, capture_output=True, universal_newlines=True).stdout)

def get_all_branches(remote:bool=True):
    remote_option = "-r"
    if not remote:
        remote_option = ""
    return common.get_list_from_string(subprocess.run([f"git branch {remote_option} | egrep -v \"(^\*|master)\""], shell=True, capture_output=True, universal_newlines=True).stdout)

def get_stale_branches(days_since_updated:int=90):
    all_remote_branches = get_all_branches()
    stale_branches = []
    for branch in all_remote_branches:
        if subprocess.run([f"git log -1 --since={days_since_updated}.days -s {branch}"], shell=True, capture_output=True, universal_newlines=True).stdout == "":
            stale_branches.append(branch)
    remote_name = get_remote_name()
    return common.replace_every_string(f"{remote_name}/", "", stale_branches) 

def delete_branch(branch, local, remote=False, remote_name="origin", force=False):
    if local:
        delete_option = "-d"
        if force:
            delete_option = "-D"
        subprocess.run([f"git branch {delete_option} {branch}"], shell=True, universal_newlines=True)

    if remote:
        subprocess.run([f"git push {remote_name} :{branch}"], shell=True, universal_newlines=True)

def merge():
    subprocess.run(["git merge"], shell=True)

def pull_all_branches():
    fetch()
    current_branch = get_current_branch()
    all_remote_branches = common.replace_every_string(f"{get_remote_name()}/", "", get_all_branches())
    for remote_branch in all_remote_branches:
        checkout(remote_branch)
        merge() 
    checkout(current_branch)

""" A list of branches not pushed you need to keep them safe 
"""
def get_local_branches_not_in_remote():
    local_branches = set(get_all_branches(remote=False))
    remote_branches = set(common.replace_every_string(f"{get_remote_name()}/", "", get_all_branches(remote=True)))
    return local_branches - remote_branches

""" A list of branches that pushed and can be deleted safely 
    (enhancement needs to be done to make sure that local matches the reference/hash of the tracking remote)
"""
def get_local_branches_that_in_remote():
    local_branches = set(get_all_branches(remote=False))
    remote_branches = set(common.replace_every_string(f"{get_remote_name()}/", "", get_all_branches(remote=True)))
    return local_branches & remote_branches



from setuptools import setup, find_packages

setup(
    name='git power ext',
    version='0.1.63',
    author="Mohamed Farouk",
    author_email="mohamed.farouk173@gmail.com",
    packages=find_packages(),
    install_requires=[
        'Click',
    ],
    entry_points={
        "console_scripts": [
            "git-power-ext = powerext.git_power_ext:cli",
        ]
    }
)